@@ -3,7 +3,7 @@
 #' spatially or temporally aggregate stars object, returning a data cube with lower spatial or temporal resolution
 #' 
 #' @param x object of class \code{stars} with information to be aggregated
-#' @param by object of class \code{sf} or \code{sfc} for spatial aggregation, for temporal aggregation a vector with time values (\code{Date}, \code{POSIXct}, or \code{PCICt}) that is interpreted as a sequence of left-closed, right-open time intervals or a string like "months", "5 days" or the like (see \link{cut.POSIXt}), or a function that cuts time into intervals; if by is an object of class \code{stars}, it is converted to sfc by \code{st_as_sfc(by, as_points = FALSE)} thus ignoring its time component. Note: each pixel is assigned to only a single group (in the order the groups occur) so non-overlapping spatial features and temporal windows are recommended.
+#' @param by object of class \code{sf} or \code{sfc} for spatial aggregation, for temporal aggregation a vector with time values (\code{Date}, \code{POSIXct}, or \code{CFTime}) that is interpreted as a sequence of left-closed, right-open time intervals or a string like "months", "5 days" or the like (see \link{cut.POSIXt}), or a function that cuts time into intervals; if by is an object of class \code{stars}, it is converted to sfc by \code{st_as_sfc(by, as_points = FALSE)} thus ignoring its time component. Note: each pixel is assigned to only a single group (in the order the groups occur) so non-overlapping spatial features and temporal windows are recommended.
 #' @param FUN aggregation function, such as \code{mean}
 #' @param ... arguments passed on to \code{FUN}, such as \code{na.rm=TRUE}
 #' @param drop logical; ignored
@@ -12,7 +12,7 @@
 #' @param left.open logical; used for time intervals, see \link{findInterval} and \link{cut.POSIXt}
 #' @param as_points see \link[stars]{st_as_sf}: shall raster pixels be taken as points, or small square polygons?
 #' @param exact logical; if \code{TRUE}, use \link[exactextractr]{coverage_fraction} to compute exact overlap fractions of polygons with raster cells
-#' @seealso \link[sf]{aggregate}, \link{st_interpolate_aw}, \link{st_extract}, https://github.com/r-spatial/stars/issues/317
+#' @seealso \link[sf]{aggregate}, \link[sf]{st_interpolate_aw}, \link{st_extract}, https://github.com/r-spatial/stars/issues/317
 #' @export
 #' @aliases aggregate
 #' @examples
@@ -72,7 +72,7 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 		left.open = FALSE, exact = FALSE) {
 
 	fn_name = substr(deparse1(substitute(FUN)), 1, 20)
-	classes = c("sf", "sfc", "POSIXct", "Date", "PCICt", "character", "function", "stars")
+	classes = c("sf", "sfc", "POSIXct", "Date", "character", "function", "stars")
 	if (!is.function(by) && !inherits(by, classes))
 		stop(paste("currently, only `by' arguments of class", 
 			paste(classes, collapse= ", "), "supported"))
@@ -106,7 +106,7 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 		new_dim = c(prod(dim(x)[1:2]), prod(dim(x)[-(1:2)]))
 		out_dim = c(ncol(m), dim(x)[-(1:2)])
 		if (isTRUE(list(...)$na.rm))
-			x = lapply(x, function(y) { y[is.na(y)] = 0.0; y })
+			x = st_as_stars(lapply(x, function(y) { y[is.na(y)] = 0.0; y }), dimensions = st_dimensions(x))
 		agg = lapply(x, function(a) array(t(m) %*% array(a, dim = new_dim), dim = out_dim))
 		# %*% dropped units, so to propagate units, if present we need to copy (mean/sum):
 		d = create_dimensions(append(setNames(list(create_dimension(values = by)), geom),
@@ -119,6 +119,7 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 		return(st_as_stars(agg, dimensions = d))
 	}
 
+	values = NULL
 	drop_y = FALSE
 	grps = if (inherits(by, c("sf", "sfc"))) {
 			x = if (has_raster(x)) {
@@ -153,11 +154,17 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 					i = as.factor(i)
 				by = levels(i)
 			} else if (inherits(by, "character")) {
-				i = cut(values, by, right = left.open)
-				by = if (inherits(values, "Date"))
-						as.Date(levels(i))
-					else
-						as.POSIXct(levels(i))
+				if (inherits(values, "CFTime")) {
+					i  = values$cut(by)
+					by = levels(i)
+					new_time = attr(i, "CFTime")
+				} else {
+					i = cut(values, by, right = left.open)
+					by = if (inherits(values, "Date"))
+							as.Date(levels(i))
+						else
+							as.POSIXct(levels(i))
+				}
 			} else {
 				if (!inherits(values, class(by)))
 					warning(paste0('argument "by" is of a different class (', class(by)[1], 
@@ -171,17 +178,21 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 	d = st_dimensions(x)
 	dims = dim(d)
 
-	agr_grps = function(x, grps, uq, FUN, ...) { 
-		do.call(rbind, lapply(uq, function(i) {
+	agr_grps = function(x, grps, uq, FUN, bind, ...) { 
+		do.call(bind, lapply(uq, function(i) {
 				sel <- which(grps == i)
 				if (!isTRUE(any(sel)))
-					rep(NA_real_, ncol(x))
+					NA_real_
 				else
 					apply(x[sel, , drop = FALSE], 2, FUN, ...)
 			}
 		))
 	}
 
+	bind = if (length(FUN(1:10, ...)) > 1)
+			cbind
+		else
+			rbind
 	# rearrange:
 	x = structure(x, dimensions = NULL, class = NULL) # unclass
 	newdims = c(prod(dims[1:ndims]), prod(dims[-(1:ndims)]))
@@ -196,17 +207,22 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 					NULL
 			} else
 				NULL
-		x[[i]] = agr_grps(a, grps, seq_along(by), FUN, ...)
+		x[[i]] = agr_grps(a, grps, seq_along(by), FUN, bind, ...)
 		if (is.numeric(x[[i]]) && !is.null(u))
 			x[[i]] = units::set_units(x[[i]], u, mode = "standard")
 	}
 
 	# reconstruct dimensions table:
-	d[[1]] = create_dimension(values = by)
-	names(d)[1] = if (is.function(by) || inherits(by, c("POSIXct", "Date", "PCICt", "function")))
-			"time"
-		else
-			geom
+	if (!is.null(values) && inherits(values, "CFTime")) {
+		d[[1]] = create_dimension(refsys = "CFTime", values = new_time)
+		names(d)[1] <- "time"
+	} else {
+		d[[1]] = create_dimension(values = by)
+		names(d)[1] = if (is.function(by) || inherits(by, c("POSIXct", "Date", "function")))
+				"time"
+			else
+				geom
+	}
 	if (drop_y)
 		d = d[-2] # y
 	
