@@ -498,7 +498,10 @@ merge.stars_proxy = function(x, y, ..., name = "attributes") {
 			if (!is.null(r <- get_range(lst[[4]]))) {
 				attr(x, "dimensions")[[ix]]$from = r[1]
 				attr(x, "dimensions")[[ix]]$to = r[2]
-				if(!is.null(attr(x, "dimensions")[[ix]]$values)) {
+				if (inherits(attr(x, "dimensions")[[ix]]$values, "CFTime")) {
+					idx = CFtime::indexOf(lst[[4]], attr(x, "dimensions")[[ix]]$values)
+					attr(x, "dimensions")[[ix]]$values = attr(idx, "CFTime")
+				} else if(!is.null(attr(x, "dimensions")[[ix]]$values)) {
 					attr(x, "dimensions")[[ix]]$values <- 
 						attr(x, "dimensions")[[ix]]$values[r[1]:r[2]]
 				}
@@ -547,12 +550,14 @@ merge.stars_proxy = function(x, y, ..., name = "attributes") {
 
 # shrink bbox with e * width in each direction
 bb_shrink = function(bb, e) {
+	stopifnot(inherits(bb, "bbox"))
 	dx = diff(bb[c("xmin", "xmax")])
 	dy = diff(bb[c("ymin", "ymax")])
 	st_bbox(setNames(c(bb["xmin"] + e * dx, 
 		bb["ymin"] + e * dy, 
 		bb["xmax"] - e * dx, 
-		bb["ymax"] - e * dy), c("xmin", "ymin", "xmax", "ymax")))
+		bb["ymax"] - e * dy), c("xmin", "ymin", "xmax", "ymax")),
+		crs = st_crs(bb))
 }
 
 #' @name st_crop
